@@ -1,3 +1,22 @@
+"""
+Copyright (c) 2022 Poutyne and all respective contributors.
+
+Each contributor holds copyright over their respective contributions. The project versioning (Git)
+records all such contribution source information.
+
+This file is part of Poutyne.
+
+Poutyne is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
+License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later
+version.
+
+Poutyne is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
+of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
+
+You should have received a copy of the GNU Lesser General Public License along with Poutyne. If not, see
+<https://www.gnu.org/licenses/>.
+"""
+
 import itertools
 from typing import Dict, Callable
 
@@ -8,7 +27,7 @@ from .color_formatting import ColorProgress
 class ProgressionCallback(Callback):
     """
     Default progression callback used in :class:`~poutyne.Model`. You can use the ``progress_options``
-    in :class:`~poutyne.Model` instead of instanciating this callback. If you choose to use this callback
+    in :class:`~poutyne.Model` instead of instantiating this callback. If you choose to use this callback
     anyway, make sure to pass ``verbose=False`` to :func:`~poutyne.Model.fit()` or
     :func:`~poutyne.Model.fit_generator()`.
 
@@ -41,28 +60,36 @@ class ProgressionCallback(Callback):
 
     EVERY_N_STEPS_CHOICES = ['all', 'none']
 
-    def __init__(self,
-                 *,
-                 coloring=True,
-                 progress_bar=True,
-                 equal_weights=False,
-                 show_on_valid=True,
-                 show_every_n_train_steps='all',
-                 show_every_n_valid_steps='all',
-                 show_every_n_test_steps='all') -> None:
+    def __init__(
+        self,
+        *,
+        coloring=True,
+        progress_bar=True,
+        equal_weights=False,
+        show_on_valid=True,
+        show_every_n_train_steps='all',
+        show_every_n_valid_steps='all',
+        show_every_n_test_steps='all',
+    ) -> None:
         super().__init__()
         self.color_progress = ColorProgress(coloring=coloring)
         self.progress_bar = progress_bar
         self.equal_weights = equal_weights
         self.show_on_valid = show_on_valid
-        self.step_times_weighted_sum = 0.
+        self.step_times_weighted_sum = 0.0
 
-        assert isinstance(show_every_n_train_steps, int) or \
-            show_every_n_train_steps in ProgressionCallback.EVERY_N_STEPS_CHOICES
-        assert isinstance(show_every_n_valid_steps, int) or \
-            show_every_n_valid_steps in ProgressionCallback.EVERY_N_STEPS_CHOICES
-        assert isinstance(show_every_n_test_steps, int) or \
-            show_every_n_test_steps in ProgressionCallback.EVERY_N_STEPS_CHOICES
+        assert (
+            isinstance(show_every_n_train_steps, int)
+            or show_every_n_train_steps in ProgressionCallback.EVERY_N_STEPS_CHOICES
+        )
+        assert (
+            isinstance(show_every_n_valid_steps, int)
+            or show_every_n_valid_steps in ProgressionCallback.EVERY_N_STEPS_CHOICES
+        )
+        assert (
+            isinstance(show_every_n_test_steps, int)
+            or show_every_n_test_steps in ProgressionCallback.EVERY_N_STEPS_CHOICES
+        )
         self.show_every_n_train_steps = show_every_n_train_steps
         self.show_every_n_valid_steps = show_every_n_valid_steps
         self.show_every_n_test_steps = show_every_n_test_steps
@@ -75,6 +102,7 @@ class ProgressionCallback(Callback):
         self._train_steps = self.params['steps']
         self._valid_steps = self.params.get('valid_steps')
         self._test_steps = self.params['steps']
+        self._predict_steps = self.params['steps']
 
     def _set_progress_bar(self):
         if self.progress_bar and self.steps is not None:
@@ -92,7 +120,7 @@ class ProgressionCallback(Callback):
 
     def on_valid_begin(self, logs: Dict) -> None:
         if self.show_on_valid:
-            self.step_times_weighted_sum = 0.
+            self.step_times_weighted_sum = 0.0
 
             self.metrics = ['loss'] + self.model.metrics_names
             self.steps = self._valid_steps
@@ -102,7 +130,7 @@ class ProgressionCallback(Callback):
             self.color_progress.on_valid_begin()
 
     def on_test_begin(self, logs: Dict) -> None:
-        self.step_times_weighted_sum = 0.
+        self.step_times_weighted_sum = 0.0
 
         self.metrics = ['loss'] + self.model.metrics_names
         self.steps = self._test_steps
@@ -111,8 +139,18 @@ class ProgressionCallback(Callback):
 
         self.color_progress.on_test_begin()
 
+    def on_predict_begin(self, logs: Dict) -> None:
+        self.step_times_weighted_sum = 0.0
+
+        self.metrics = []
+        self.steps = self._predict_steps
+
+        self._set_progress_bar()
+
+        self.color_progress.on_predict_begin()
+
     def on_epoch_begin(self, epoch_number: int, logs: Dict) -> None:
-        self.step_times_weighted_sum = 0.
+        self.step_times_weighted_sum = 0.0
         self.epoch_number = epoch_number
         self.steps = self._train_steps
 
@@ -125,10 +163,12 @@ class ProgressionCallback(Callback):
         epoch_total_time = logs['time']
 
         metrics_str = self._get_metrics_string(logs)
-        self.color_progress.on_epoch_end(total_time=epoch_total_time,
-                                         train_last_steps=self.train_last_step,
-                                         valid_last_steps=self.valid_last_step,
-                                         metrics_str=metrics_str)
+        self.color_progress.on_epoch_end(
+            total_time=epoch_total_time,
+            train_last_steps=self.train_last_step,
+            valid_last_steps=self.valid_last_step,
+            metrics_str=metrics_str,
+        )
 
     def on_test_end(self, logs: Dict) -> None:
         test_total_time = logs['time']
@@ -136,20 +176,29 @@ class ProgressionCallback(Callback):
 
         self._end_progress(logs, test_total_time, progress_fun)
 
+    def on_predict_end(self, logs: Dict) -> None:
+        predict_total_time = logs['time']
+        progress_fun = self.color_progress.on_predict_end
+
+        self._end_progress(logs, predict_total_time, progress_fun)
+
     def _test_show_batch_end(self, show_every_n_steps_flag, batch_number):
-        return show_every_n_steps_flag == 'all' or \
-            (isinstance(show_every_n_steps_flag, int) and batch_number % show_every_n_steps_flag == 0)
+        return show_every_n_steps_flag == 'all' or (
+            isinstance(show_every_n_steps_flag, int) and batch_number % show_every_n_steps_flag == 0
+        )
 
     def on_train_batch_end(self, batch_number: int, logs: Dict) -> None:
         train_step_times_rate = self._compute_step_times_rate(batch_number, logs)
         progress_batch_end_fun = self.color_progress.on_train_batch_end
 
         do_print = self._test_show_batch_end(self.show_every_n_train_steps, batch_number)
-        self._batch_end_progress(logs=logs,
-                                 step_times_rate=train_step_times_rate,
-                                 batch_number=batch_number,
-                                 func=progress_batch_end_fun,
-                                 do_print=do_print)
+        self._batch_end_progress(
+            logs=logs,
+            step_times_rate=train_step_times_rate,
+            batch_number=batch_number,
+            func=progress_batch_end_fun,
+            do_print=do_print,
+        )
         self.train_last_step = batch_number
 
     def on_valid_batch_end(self, batch_number: int, logs: Dict) -> None:
@@ -158,11 +207,13 @@ class ProgressionCallback(Callback):
             progress_batch_end_fun = self.color_progress.on_valid_batch_end
 
             do_print = self._test_show_batch_end(self.show_every_n_valid_steps, batch_number)
-            self._batch_end_progress(logs=logs,
-                                     step_times_rate=valid_step_times_rate,
-                                     batch_number=batch_number,
-                                     func=progress_batch_end_fun,
-                                     do_print=do_print)
+            self._batch_end_progress(
+                logs=logs,
+                step_times_rate=valid_step_times_rate,
+                batch_number=batch_number,
+                func=progress_batch_end_fun,
+                do_print=do_print,
+            )
 
         self.valid_last_step = batch_number
 
@@ -171,18 +222,35 @@ class ProgressionCallback(Callback):
         progress_batch_end_fun = self.color_progress.on_test_batch_end
 
         do_print = self._test_show_batch_end(self.show_every_n_test_steps, batch_number)
-        self._batch_end_progress(logs=logs,
-                                 step_times_rate=test_step_times_rate,
-                                 batch_number=batch_number,
-                                 func=progress_batch_end_fun,
-                                 do_print=do_print)
+        self._batch_end_progress(
+            logs=logs,
+            step_times_rate=test_step_times_rate,
+            batch_number=batch_number,
+            func=progress_batch_end_fun,
+            do_print=do_print,
+        )
+
+    def on_predict_batch_end(self, batch_number: int, logs: Dict) -> None:
+        predict_step_times_rate = self._compute_step_times_rate(batch_number, logs)
+        progress_batch_end_fun = self.color_progress.on_predict_batch_end
+
+        do_print = self._test_show_batch_end(self.show_every_n_test_steps, batch_number)
+        self._batch_end_progress(
+            logs=logs,
+            step_times_rate=predict_step_times_rate,
+            batch_number=batch_number,
+            func=progress_batch_end_fun,
+            do_print=do_print,
+        )
 
     def _get_metrics_string(self, logs: Dict) -> str:
-        train_metrics_str_gen = ('{}: {:f}'.format(k, logs[k]) for k in self.metrics if logs.get(k) is not None)
-        val_metrics_str_gen = ('{}: {:f}'.format('val_' + k, logs['val_' + k]) for k in self.metrics
-                               if logs.get('val_' + k) is not None)
-        test_metrics_str_gen = ('{}: {:f}'.format('test_' + k, logs['test_' + k]) for k in self.metrics
-                                if logs.get('test_' + k) is not None)
+        train_metrics_str_gen = (f'{k}: {logs[k]:f}' for k in self.metrics if logs.get(k) is not None)
+        val_metrics_str_gen = (
+            f"{'val_' + k}: {logs['val_' + k]:f}" for k in self.metrics if logs.get('val_' + k) is not None
+        )
+        test_metrics_str_gen = (
+            f"{'test_' + k}: {logs['test_' + k]:f}" for k in self.metrics if logs.get('test_' + k) is not None
+        )
         return ', '.join(itertools.chain(train_metrics_str_gen, val_metrics_str_gen, test_metrics_str_gen))
 
     def _compute_step_times_rate(self, batch_number: int, logs: Dict) -> float:
@@ -205,19 +273,22 @@ class ProgressionCallback(Callback):
         else:
             func(total_time=total_time, steps=self.last_step, metrics_str=metrics_str)
 
-    def _batch_end_progress(self, *, logs: Dict, step_times_rate: float, batch_number: int, func: Callable,
-                            do_print: bool) -> None:
+    def _batch_end_progress(
+        self, *, logs: Dict, step_times_rate: float, batch_number: int, func: Callable, do_print: bool
+    ) -> None:
         """
         Update the progress at the end of train, valid or test batch.
         """
         metrics_str = self._get_metrics_string(logs)
         if self.steps is not None:
             remaining_time = step_times_rate * (self.steps - batch_number)
-            func(remaining_time=remaining_time,
-                 batch_number=batch_number,
-                 metrics_str=metrics_str,
-                 steps=self.steps,
-                 do_print=do_print)
+            func(
+                remaining_time=remaining_time,
+                batch_number=batch_number,
+                metrics_str=metrics_str,
+                steps=self.steps,
+                do_print=do_print,
+            )
         else:
             func(remaining_time=step_times_rate, batch_number=batch_number, metrics_str=metrics_str, do_print=do_print)
             self.last_step = batch_number
@@ -230,8 +301,10 @@ class EpochProgressionCallback(Callback):
         super().__init__()
         self.color_progress = ColorProgress(coloring=coloring)
 
-        assert isinstance(show_every_n_epochs, int) or \
-            show_every_n_epochs in EpochProgressionCallback.EVERY_N_EPOCHS_CHOICES
+        assert (
+            isinstance(show_every_n_epochs, int)
+            or show_every_n_epochs in EpochProgressionCallback.EVERY_N_EPOCHS_CHOICES
+        )
         self.show_every_n_epochs = show_every_n_epochs
 
     def set_params(self, params: Dict):
@@ -244,8 +317,9 @@ class EpochProgressionCallback(Callback):
         self.epochs = self.params['epochs']
 
     def _test_show_epoch(self, epoch_number):
-        return self.show_every_n_epochs != 'none' and \
-            (self.show_every_n_epochs == 'all' or epoch_number % self.show_every_n_epochs == 0)
+        return self.show_every_n_epochs != 'none' and (
+            self.show_every_n_epochs == 'all' or epoch_number % self.show_every_n_epochs == 0
+        )
 
     def on_epoch_begin(self, epoch_number: int, logs: Dict) -> None:
         if self._test_show_epoch(epoch_number):
@@ -253,13 +327,16 @@ class EpochProgressionCallback(Callback):
 
     def on_epoch_end(self, epoch_number: int, logs: Dict) -> None:
         if self._test_show_epoch(epoch_number):
-            self.color_progress.on_epoch_end(total_time=logs['time'],
-                                             train_last_steps=self._train_steps,
-                                             valid_last_steps=self._valid_steps,
-                                             metrics_str=self._get_metrics_string(logs))
+            self.color_progress.on_epoch_end(
+                total_time=logs['time'],
+                train_last_steps=self._train_steps,
+                valid_last_steps=self._valid_steps,
+                metrics_str=self._get_metrics_string(logs),
+            )
 
     def _get_metrics_string(self, logs: Dict) -> str:
-        train_metrics_str_gen = ('{}: {:f}'.format(k, logs[k]) for k in self.metrics if logs.get(k) is not None)
-        val_metrics_str_gen = ('{}: {:f}'.format('val_' + k, logs['val_' + k]) for k in self.metrics
-                               if logs.get('val_' + k) is not None)
+        train_metrics_str_gen = (f'{k}: {logs[k]:f}' for k in self.metrics if logs.get(k) is not None)
+        val_metrics_str_gen = (
+            f"{'val_' + k}: {logs['val_' + k]:f}" for k in self.metrics if logs.get('val_' + k) is not None
+        )
         return ', '.join(itertools.chain(train_metrics_str_gen, val_metrics_str_gen))
